@@ -44,6 +44,7 @@ L2tpWidget::L2tpWidget(const NetworkManager::VpnSetting::Ptr &setting, QWidget*
     connect(m_ui->btnPPPSettings, &QPushButton::clicked, this, &L2tpWidget::showPpp);
 
     connect(m_ui->gateway, &QLineEdit::textChanged, this, &L2tpWidget::slotWidgetChanged);
+    connect(m_ui->cbUseCertificate, &QCheckBox::stateChanged, this, &L2tpWidget::certStateChanged);
 
     KAcceleratorManager::manage(this);
 
@@ -65,14 +66,12 @@ void L2tpWidget::loadConfig(const NetworkManager::Setting::Ptr &setting)
 
     const NMStringMap data = m_setting->data();
 
-    const QString gateway = data.value(NM_L2TP_KEY_GATEWAY);
-    if (!gateway.isEmpty()) {
-        m_ui->gateway->setText(gateway);
+    if (data.contains(NM_L2TP_KEY_GATEWAY)) {
+        m_ui->gateway->setText(data.value(NM_L2TP_KEY_GATEWAY));
     }
 
-    const QString user = data.value(NM_L2TP_KEY_USER);
-    if (!user.isEmpty()) {
-        m_ui->username->setText(user);
+    if (data.contains(NM_L2TP_KEY_USER)) {
+        m_ui->username->setText(data.value(NM_L2TP_KEY_USER));
     }
 
     const NetworkManager::Setting::SecretFlags userPassType = static_cast<NetworkManager::Setting::SecretFlags>(data.value(NM_L2TP_KEY_PASSWORD"-flags").toInt());
@@ -84,9 +83,24 @@ void L2tpWidget::loadConfig(const NetworkManager::Setting::Ptr &setting)
         m_ui->password->setPasswordOption(PasswordField::AlwaysAsk);
     }
 
-    const QString domain = data.value(NM_L2TP_KEY_DOMAIN);
-    if (!domain.isEmpty()) {
-        m_ui->domain->setText(domain);
+    if (data.contains(NM_L2TP_KEY_DOMAIN)) {
+        m_ui->domain->setText(data.value(NM_L2TP_KEY_DOMAIN));
+    }
+
+    if (data.contains(NM_L2TP_KEY_CERT_CA)) {
+        m_ui->urCACertificate->setText(data.value(NM_L2TP_KEY_CERT_CA));
+    }
+
+    if (data.contains(NM_L2TP_KEY_CERT_PUB)) {
+        m_ui->urCertificate->setText(data.value(NM_L2TP_KEY_CERT_PUB));
+    }
+
+    if (data.contains(NM_L2TP_KEY_CERT_KEY)) {
+        m_ui->urPrivateKey->setText(data.value(NM_L2TP_KEY_CERT_KEY));
+    }
+
+    if (data.value(NM_L2TP_KEY_USE_CERT) == QLatin1String("yes")) {
+        m_ui->cbUseCertificate->setChecked(true);
     }
 
     loadSecrets(setting);
@@ -112,34 +126,67 @@ QVariantMap L2tpWidget::setting() const
     NMStringMap data;
     if (!m_tmpAdvancedSetting.isNull()) {
         data = m_tmpAdvancedSetting->data();
+    } else {
+        // retrieve the settings if the dialog has not been opened
+        QScopedPointer<L2tpAdvancedWidget> adv(new L2tpAdvancedWidget(m_setting, Q_NULLPTR));
+        data = adv->setting();
     }
+
     if (!m_tmpPppSetting.isNull()) {
         data.unite(m_tmpPppSetting->data());
+    } else {
+        // retrieve the settings if the dialog has not been opened
+        QScopedPointer<L2tpPPPWidget> ppp(new L2tpPPPWidget(m_setting, Q_NULLPTR));
+        data.unite(ppp->setting());
     }
+
     NMStringMap secrets;
 
     if (!m_ui->gateway->text().isEmpty()) {
         data.insert(NM_L2TP_KEY_GATEWAY, m_ui->gateway->text());
     }
 
-    if (!m_ui->username->text().isEmpty()) {
-        data.insert(NM_L2TP_KEY_USER, m_ui->username->text());
-    }
+    if (m_ui->cbUseCertificate->isChecked()) {
+        data.insert(NM_L2TP_KEY_USE_CERT, "yes");
 
-    if (m_ui->password->isEnabled() && !m_ui->password->text().isEmpty()) {
-        secrets.insert(NM_L2TP_KEY_PASSWORD, m_ui->password->text());
-    }
+        if (!m_ui->urCACertificate->text().isEmpty()) {
+            data.insert(NM_L2TP_KEY_CERT_CA, m_ui->urCACertificate->text());
+        }
 
-    if (m_ui->password->passwordOption() == PasswordField::StoreForAllUsers) {
-        data.insert(NM_L2TP_KEY_PASSWORD"-flags", QString::number(NetworkManager::Setting::None));
-    } else if (m_ui->password->passwordOption() == PasswordField::StoreForUser) {
-        data.insert(NM_L2TP_KEY_PASSWORD"-flags", QString::number(NetworkManager::Setting::AgentOwned));
-    } else { // SettingWidget::EnumPasswordStorageType::Store
-        data.insert(NM_L2TP_KEY_PASSWORD"-flags", QString::number(NetworkManager::Setting::NotSaved));
-    }
+        if (!m_ui->urCertificate->text().isEmpty()) {
+            data.insert(NM_L2TP_KEY_CERT_PUB, m_ui->urCertificate->text());
+        }
 
-    if (!m_ui->domain->text().isEmpty()) {
-        data.insert(NM_L2TP_KEY_DOMAIN, m_ui->domain->text());
+        if (!m_ui->urPrivateKey->text().isEmpty()) {
+            data.insert(NM_L2TP_KEY_CERT_KEY, m_ui->urPrivateKey->text());
+        }
+
+        data.insert(NM_L2TP_KEY_PASSWORD"-flags", QString::number(NetworkManager::Setting::NotRequired));
+
+    } else {
+
+        if (!m_ui->username->text().isEmpty()) {
+            data.insert(NM_L2TP_KEY_USER, m_ui->username->text());
+        }
+
+        if (m_ui->password->isEnabled() && !m_ui->password->text().isEmpty()) {
+            secrets.insert(NM_L2TP_KEY_PASSWORD, m_ui->password->text());
+        }
+
+        switch (m_ui->password->passwordOption()) {
+        case PasswordField::StoreForAllUsers:
+            data.insert(NM_L2TP_KEY_PASSWORD"-flags", QString::number(NetworkManager::Setting::None));
+            break;
+        case PasswordField::StoreForUser:
+            data.insert(NM_L2TP_KEY_PASSWORD"-flags", QString::number(NetworkManager::Setting::AgentOwned));
+            break;
+        default:
+            data.insert(NM_L2TP_KEY_PASSWORD"-flags", QString::number(NetworkManager::Setting::NotSaved));
+        };
+
+        if (!m_ui->domain->text().isEmpty()) {
+            data.insert(NM_L2TP_KEY_DOMAIN, m_ui->domain->text());
+        }
     }
 
     setting.setData(data);
@@ -213,3 +260,22 @@ bool L2tpWidget::isValid() const
 {
     return !m_ui->gateway->text().isEmpty();
 }
+
+void L2tpWidget::certStateChanged()
+{
+    if (m_ui->cbUseCertificate->isChecked()) {
+        m_ui->urCACertificate->setEnabled(true);
+        m_ui->urCertificate->setEnabled(true);
+        m_ui->urPrivateKey->setEnabled(true);
+        m_ui->username->setEnabled(false);
+        m_ui->password->setEnabled(false);
+        m_ui->domain->setEnabled(false);
+    } else {
+        m_ui->urCACertificate->setEnabled(false);
+        m_ui->urCertificate->setEnabled(false);
+        m_ui->urPrivateKey->setEnabled(false);
+        m_ui->username->setEnabled(true);
+        m_ui->password->setEnabled(true);
+        m_ui->domain->setEnabled(true);
+    }
+}
